/*
 * Licensed Materials - Property of IBM Corp.
 * IBM UrbanCode Build
 * IBM UrbanCode Deploy
 * IBM UrbanCode Release
 * IBM AnthillPro
 * (c) Copyright IBM Corporation 2002, 2014. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights - Use, duplication or disclosure restricted by
 * GSA ADP Schedule Contract with IBM Corp.
 */
import com.urbancode.anthill3.AHPTool
import groovy.xml.StreamingMarkupBuilder;

import java.util.regex.Matcher
import java.util.regex.Pattern

def ahptool = new AHPTool();


class FileSet {
    def isWindows = (System.getProperty('os.name') =~ /(?i)windows/).find()
    def base
    def includes = []
    def excludes = []

    def FileSet(base) {
        if (base instanceof File) {
            this.base = base
        }
        else if (base == null) {
            this.base = new File('.').absoluteFile
        }
        else {
            this.base = new File(base)
        }
    }

    def include(antPattern) {
        forNonEmptyLines(antPattern) {
            includes << convertToPattern(it)
        }
    }

    def exclude(antPattern) {
        forNonEmptyLines(antPattern) {
            excludes << convertToPattern(it)
        }
    }

    def each(Closure closure) {
        base.eachFileRecurse { file ->
            def path = file.path.replace('\\', '/').substring(base.path.length())
            def matches = false
            for (p in includes) {
                if (path =~ p) {
                    matches = true
                    break;
                }
            }
            if (matches) {
                for (p in excludes) {
                    if (path =~ p) {
                        matches = false;
                        break;
                    }
                }
            }
            if (matches) {
                closure(file)
            }
        }
    }

    def files() {
        def list = []
        each { list << it }
        return list
    }

    private forNonEmptyLines(strings, Closure closure) {
        if (strings instanceof Collection) {
            strings.each {
                def trimmed = it?.trim()
                if (trimmed?.length() > 0) {
                    closure(it)
                }
            }
        }
        else if (strings != null) {
            forNonEmptyLines(strings.readLines(), closure)
        }
    }

    private convertToPattern(antPattern) {
        // trim leading / character from pattern
        def pattern = antPattern.startsWith('/') ? antPattern : '/'+antPattern

        // deal with special regex-characters that should be interpreted as literals
        '\\.+[]^${}|()'.toCharArray().each{ c ->
            pattern = pattern.replace(''+c, '\\'+c)
        }
        pattern = pattern.replace('?', '.') // ? is a single-char wildcard

        // deal with ant-style wildcards
        StringBuffer result = new StringBuffer()
        result.append("^")
        def m = (pattern =~ '\\*\\*/|\\*\\*|\\*')
        while (m) {
            def token = m.group()
            def replacement;
            if (token == '**/') {
                replacement = '.*(?<=/)'
            }
            else if (token == '**') {
                replacement = '.*'
            }
            else {
                replacement = '[^/]*'
            }
            m.appendReplacement(result, Matcher.quoteReplacement(replacement))
        }
        m.appendTail(result)
        result.append("\$")
        def flags = 0
        if (isWindows) {
            flags |= Pattern.CASE_INSENSITIVE
        }
        return Pattern.compile(result.toString(), flags)
    }
}


def uploadResult (reportFiles,reportName,ahptool,truncate) {
    def testsuiteName
    def reportXmlBuilder = new StreamingMarkupBuilder().bind{
        "test-report"("name":reportName,
        "type": 'JUnit',
        "job-id": System.getenv('AH_JOB_ID')) {

            reportFiles.each {
                xml = it
                def rootNode = new XmlParser().parse(xml)
                if (rootNode.name().equals("testsuite")) {
                    def testsuite = rootNode
                    if (truncate) {
                        testsuiteName = testsuite.@name.substring(testsuite.@name.lastIndexOf('.') + 1)
                    }
                    else {
                        testsuiteName = testsuite.@name
                    }
                    "test-suite"("name": testsuiteName) {
                        testsuite.testcase.each { testcase ->
                            String result, name;
                            def time;
                            if (testcase.failure || testcase.error) {
                                result = 'failure'
                            }
                            else if (testcase.skipped) {
                                result = 'skipped';
                            }
                            else {
                                result = 'success'
                            }
                            if (testcase.@classname){
                                name = testcase.@classname + ' ' +  testcase.@name
                            }
                            else {
                                name = testcase.@name
                            }
                            time = testcase.@time
                            if (!time) {
                                time = "0"
                            }
                            time = time.replaceAll(',', '')
                            time = (int) (Double.parseDouble(time) * 1000)
                            "test" ("name": name, "result": result ,"time": time) {
                                if (result == 'skipped') {
                                    def msg = ''
                                    testcase.skipped.findAll{
                                        msg += "Message: "+it.@message +'\n';
                                        msg += "Text: "+it.text() + '\n';
                                    }
                                    "message"(msg);
                                }
                                else if (result != 'success') {
                                    def errorMsg = ""
                                    testcase.error.findAll { error -> errorMsg += 'Error Message: ' + error.@message + ' Error Text: ' +  error.text() }
                                    testcase.failure.findAll { error -> errorMsg += 'Error Message: ' + error.@message + ' Error Text: ' + error.text() }
                                    "message"(errorMsg)
                                }
                            }
                        }
                    }
                }
                else {
                    rootNode.testsuite.each { testsuite ->
                        if (truncate) {
                            testsuiteName = testsuite.@name.substring(testsuite.@name.lastIndexOf('.') + 1)
                        }
                        else {
                            testsuiteName = testsuite.@name
                        }
                        "test-suite"("name": testsuiteName) {
                            testsuite.testcase.each { testcase ->
                                String result, name;
                                def time;
                                if (testcase.failure || testcase.error) {
                                    result = 'failure'
                                }
                                else if (testcase.skipped) {
                                    result = 'skipped';
                                }
                                else {
                                    result = 'success'
                                }

                                if (testcase.@classname){
                                    name = testcase.@classname + ' ' +  testcase.@name
                                }
                                else {
                                    name =  testcase.@name
                                }

                                time = testcase.@time
                                if (!time) {
                                    time = "0"
                                }
                                time = time.replaceAll(',', '')
                                time = (int) (Double.parseDouble(time) * 1000)
                                "test" ("name": name, "result": result ,"time": time) {
                                    if (result == 'skipped') {
                                        def msg = ''
                                        testcase.skipped.findAll{
                                            msg += "Message: "+it.@message +'\n';
                                            msg += "Text: "+it.text() + '\n';
                                        }
                                        "message"(msg);
                                    }
                                    else if (result != 'success') {
                                        def errorMsg = ""
                                        testcase.error.findAll { error -> errorMsg += 'Error Message: ' + error.@message + ' Error Text: ' +  error.text() }
                                        testcase.failure.findAll { error -> errorMsg += 'Error Message: ' + error.@message + ' Error Text: ' + error.text() }
                                        "message"(errorMsg)
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    ahptool.addTests(reportName, reportXmlBuilder)
}

def props = [];
props = ahptool.getStepProperties();
def reportName = props['reportName'] ? props['reportName'] : 'JUnit Report'
def sourceDir = new File(props['sourceDir'])
def includes = props['includes']
def excludes = props['excludes']
def truncate = props['truncate']
try {
    def fileSet = new FileSet(sourceDir)
    fileSet.include(includes)
    fileSet.exclude(excludes)

    def reportFiles = fileSet.files()
    println 'Base Directory: ' +  sourceDir.getAbsolutePath()
    println 'Include File Patterns: ' + includes
    println 'Exclude File Patterns: ' + excludes
    reportFiles.each { file->
        println 'Report File identified to be included in the JUnit Report: ' + file
    }
    if (reportFiles.size() > 0) {
        uploadResult(reportFiles,reportName,ahptool,truncate)
    }
    else {
        println 'No report files found to upload to AnthillPro'
    }
}
catch (Exception e) {
    println e
    System.exit(1)
}

